/* sc_strings.c
 * SommerCampus 2006
 * CStrings Kurs - Beispiel Bibliothek
 * Diese Bibliothek soll nur als Beispiel dienen und
 * versucht deshalb die GString Klasse der Glib Bibliothek (http://www.gtk.org/api/2.6/glib/glib-Strings.html)
 * zu kopieren/nachzumachen (nicht vollstndig)
 *
 * Licence: GPL-2 */

#include <sc_strings.h>
#include <ctype.h> /* Fr isblank(): check ob Zeichen Leerzeichen oder TAB ist */

/* diese Funktion ist als static deklarier (und hat auch kein Prototyp)
 * weil sie nur innerhalb dieser Bibliothek benutzt werden soll
 *
 * newlen ist die neue Lnge der Zeichenkette. Wenn in string genug
 * Platz gibt, dann wird keinen weiteren Speicher reserviert, sonst reserviert
 * die Funktion gengend Speicher und aktualisiert string->size
 *
 * return: 1 bei Erfolg (selbst wenn keinen Speicher reserviert werden musste)
 * 	   0 bei Misserfolg: wenn Speicher reserviert werden musste und realloc keinen
 * 	   Speicher reserviert */

static int sc_strings_alloc(sc_strings *string, size_t newlen)
{
	char *tmp;
	if(!string)
		return 0;

	if(!newlen) /* man braucht keinen Speicher */
		return 1;

	if(newlen + 1 > string->size)
	{
		/* wir brauchen mehr Speicher */
		tmp = realloc(string->str, newlen + 1);

		if(!tmp) /* realloc schlgt fehl! */
			return 0;
		
		if(tmp != string->str)
		{
			string->str = tmp;
		}

		string->size = newlen + 1;
	}

	return 1;
}



sc_strings *sc_strings_new(const char *init)
{
	size_t len;
	char *tmp;
	sc_strings *string;
	
	if(!init)
		return NULL;


	string = malloc(sizeof(sc_strings));

	if(!string)
		return NULL;
	
	len = strlen(init);

	tmp = malloc(len + 1);

	if(!tmp)
	{
		/* keinen Speicher fr die Zeichenkette */
		free(string);
		return NULL;
	}

	strcpy(tmp, init);

	string->str = tmp;
	string->len = len;
	string->size = len + 1;

	return string;
}



char *sc_strings_free(sc_strings *string, int content)
{
	char *str = NULL;
	if(!string)
		return NULL;

	if(content) 
		str = string->str;
	else
		free(string->str);

	/* nicht notwendig, aber Vorsicht ist besser als Nachsicht */

	string->str = NULL;
	string->len = string->size = 0;

	free(string);

	return str;
}



int sc_strings_append_c_string(sc_strings *string, const char *src)
{
	size_t newlen;
	
	if(!string || !src)
		return 0;

	newlen = string->len + strlen(src);

	if(!sc_strings_alloc(string, newlen)) /* Speicher konnte nicht reserviert werden */
		return 0;

	
	strcat(string->str, src);

	string->len = newlen; /* string->size wurde bereits von sc_strings_alloc aktualisiert */

	return 1;
}



int sc_strings_append(sc_strings *string, const sc_strings *src)
{
	if(!src)
		return 0;

	return sc_strings_append_c_string(string, src->str);
}



int sc_strings_delete(sc_strings *string, size_t pos, size_t len)
{
	char *tmp_pos, *tmp_new;
	size_t newlen;
	
	if(!string)
		return 0;

	if(!len)
		return 1;

	if(pos > string->len - 1)
		return 0;

	if(pos + len >= string->len)
	{
		string->str[pos] = 0;
		string->len = pos; /* neue Lnge aktualisiert */
		return 1;
	}

	tmp_pos = string->str + pos; /* tmp_pos zeigt auf die Stelle pos */
	tmp_new = tmp_pos + len; /* tmp_new zeigt auf die Stelle, die nach links verschoben werden muss */

	newlen = string->str + string->len - tmp_new + 1; /* Lnge des Stckes, das verschoeben werden muss ( + \0 Zeichen ) */

	memmove(tmp_pos, tmp_new, newlen);

	string->len -= len; /* Aktualisiere neue Lnge */

	return 1;
}



int sc_strings_cut_c_string(sc_strings *string, const char *pattern)
{
	char *tmp;
	if(!string || !pattern)
		return 0;

	tmp = strstr(string->str, pattern);

	if(!tmp) /* pattern nicht gefunden */
		return 1;

	return sc_strings_delete(string, tmp - string->str, strlen(pattern));
}



int sc_strings_cut(sc_strings *string, const sc_strings *pattern)
{
	if(!pattern)
		return 0;

	return sc_strings_cut_c_string(string, pattern->str);
}



int sc_strings_replace_c_string(sc_strings *string, const char *pattern, const char *replace)
{
	char *tmp, *tmp_move;
	size_t patt_len, rep_len, tmp_pos;

	if(!string || !pattern || !replace)
		return 0;

	tmp = strstr(string->str, pattern);
	tmp_pos = tmp - string->str; /* wenn sc_strings_alloc die Adresse von string->str ndert
					dann zeigt tmp ins Nirvana, also merke ich die Stelle, wo
					pattern in string->str anfngt und nach dem Aufruf von
					sc_strings_alloc wird tmp aktualisiert */

	if(!tmp)
		return 1;


	patt_len = strlen(pattern);
	rep_len = strlen(replace);


	if(!sc_strings_alloc(string, string->len - patt_len + rep_len)) /* keinen Speicher */
		return 0;

	tmp = string->str + tmp_pos; /* falls strings->str gendert wurde */
	tmp_move = tmp + patt_len;

	/* Verschiebe Bereich nach pattern zu der neuen richtigen Stelle (inklusive \0) */
	memmove(tmp + rep_len, tmp_move, string->str + string->len - tmp_move + 1);

	/* Kopiere replace in tmp */
	memmove(tmp, replace, rep_len);

	string->len = string->len - patt_len + rep_len;

	return 1;
}



int sc_strings_replace(sc_strings *string, const sc_strings *pattern, const sc_strings *replace)
{
	if(!pattern || !replace)
		return 0;

	return sc_strings_replace_c_string(string, pattern->str, replace->str);
}



int sc_strings_insert_c_string(sc_strings *string, size_t pos, const char *insert)
{
	char *tmp_move;
	size_t ins_len;
	
	if(!string || !insert) return 0;

	if(!*insert) /* leeres String */
		return 1;

	if(pos >= string->len)
		return sc_strings_append_c_string(string, insert);

	ins_len = strlen(insert);

	if(!sc_strings_alloc(string, string->len + ins_len)) /* nicht genug Speicher */
		return 0;

	tmp_move = string->str + pos;

	/* Verschiebe Bereich nach rechts (+ \0) */
	memmove(tmp_move + ins_len, tmp_move, string->str + string->len - tmp_move + 1);

	/* Kopiere insert an Stelle pos */
	memmove(tmp_move, insert, ins_len);

	string->len += ins_len;

	return 1;
}

int sc_strings_insert(sc_strings *string, size_t pos, const sc_strings *insert)
{
	if(!insert)
		return 0;

	return sc_strings_insert_c_string(string, pos, insert->str);
}


int sc_string_ltrim(sc_strings *string)
{
	size_t i=0;

	if(!string)
		return 0;

	while(string->str[i] && isblank(string->str[i]))
		++i;

	if(i == string->len)
	{
		/* das ganze String ist "leer" */
		string->str[0]=0;
		string->len = 0;
		return 1;
	}

	memmove(string->str, string->str + i, string->len - i + 1);

	string->len -= i;

	return 1;
}


int sc_string_rtrim(sc_strings *string)
{
	int i;

	if(!string)
		return 0;

	i = string->len - 1;

	while(i >= 0 && isblank(string->str[i]))
		--i;

	string->str[i + 1] = 0;
	string->len = i + 1;

	return 1;
}


int sc_string_trim(sc_strings* string)
{
	return sc_string_ltrim(string) && sc_string_rtrim(string);
}
