/* sc_strings.h
 * SommerCampus 2006
 * CStrings Kurs - Beispiel Bibliothek
 * Diese Bibliothek soll nur als Beispiel dienen und
 * versucht deshalb die GString Klasse der Glib Bibliothek (http://www.gtk.org/api/2.6/glib/glib-Strings.html)
 * zu kopieren/nachzumachen (nicht vollstndig)
 *
 * Licence: GPL-2 */

#ifndef SC_STRINGS_H
#define SC_STRINGS_H

/* Inkludiere string.h, stdlib.h. Nutzer unserer Bibliothek brauchen diese Dateien
 * nicht mehr selber zu inkludieren */

#include <string.h>
#include <stdlib.h>

typedef struct {
	char *str; /* Zeiger auf die dyn. Zeichenkette */
	size_t len; /* Lnge der Zeichenkette */
	size_t size; /* Anzahl Bytes, die bereits reserviert sind */
} sc_strings;

sc_strings *sc_strings_new(const char *init);

/* erzeugt ein neues sc_strings Objekt (dynamisch) mit der Zeichenkette init.
 * Wenn es keinen freien Speicher gibt, dann liefert sie NULL zurck.
 * init muss 0-terminierend sein und init != NULL */

char *sc_strings_free(sc_strings *string, int content);

/* gibt den fr das sc_string Objekt reservierten Speicher zurck.
 * Wenn content 0 ist, dann wird den Speicher fr die Zeichenkette
 * ebenfalls freigegeben und NULL ist zurckgeliefert.
 * 
 * Wenn content != 0 ist, dann wird nur den Speicher fr das sc_string Objekt
 * freigegeben, den fr die Zeichenkette nicht und einen Zeiger auf diese Zeichenkette
 * wird zurckgeliefert (muss von Hand freigegeben werden) */

int sc_strings_append_c_string(sc_strings *string, const char *src);

/* hngt C-String src in string->str an
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_append(sc_strings *string, const sc_strings *src);

/* hngt src->str in string->str an
 *
 * return: 1 bei Erfolg
 *         0 sonst */

int sc_strings_delete(sc_strings *string, size_t pos, size_t len);

/* Lscht ein Teil der Zeichenkette von string
 *
 * pos: Stelle, an der gelscht werden soll. Wenn pos > string->len - 1, dann 
 * wird nichts getan, 0 ist zurckgeliefert
 *
 * len: Anzahl der Zeichen, die ab der Stelle pos gelscht werden sollen. Wenn
 * len 0 ist, dann wird nichts getan, 1  ist zurckgeliefert.
 * Wenn pos + len >= string->len dann string->str[pos] = 0; ausgefhrt
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_cut_c_string(sc_strings *string, const char *pattern);

/* Lscht C-String pattern aus string->str
 * Wenn pattern nicht gefunden wurde, dann wird ebenfalls 1 zurckgeliefert
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_cut(sc_strings *string, const sc_strings *pattern);

/* Lscht pattern aus string
 * Wenn pattern nicht gefunden wurde, dann wird ebenfalls 1 zurckgeliefert
 * 
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_replace_c_string(sc_strings *string, const char *pattern, const char *replace);

/* Ersetzt pattern mit replace in string
 * Wenn pattern nicht gefunden wurde, dann wird ebenfalls 1 zurckgeliefert
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_replace(sc_strings *string, const sc_strings *pattern, const sc_strings *replace);

/* Ersetzt pattern mit replace in string
 * Wenn pattern nicht gefunden wurde, dann wird ebenfalls 1 zurckgeliefert
 * 
 * return: 1 bei Erfolg
 *         0 sonst */

int sc_strings_insert_c_string(sc_strings *string, size_t pos, const char *insert);

/* Fgt insert in string an der Stelle pos hinzu.
 * Wenn pos pos >= string->len, dann wird sc_strings_append_c_string ausgefhrt
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_strings_insert(sc_strings *string, size_t pos, const sc_strings *insert);

/* Fgt insert in string an der Stelle pos hinzu.
 * Wenn pos pos >= string->len, dann wird sc_strings_append_c_string ausgefhrt
 *
 * return: 1 bei Erfolg
 *         0 sonst */

int sc_string_ltrim(sc_strings *string);

/* Lscht alle vorangehende Leerzeichen vor dem ersten nicht-Leerzeichen
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_string_rtrim(sc_strings *string);

/* Lscht alle Leerzeichen nach dem letzten nicht-Leerzeichen
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

int sc_string_trim(sc_strings* string);

/* Lscht alle vorangehende und alle letzten leere Zeichen
 *
 * return: 1 bei Erfolg
 * 	   0 sonst */

#endif
